C
C NAME:DORSAL V6-3-12
C AUTHOR: MACHETEL AND GARRIDO GMD-2013
C DESCRIPTION: RESOLUTION OF MID-OCEAN RIDGE DYNAMICS AND TEMPERATURE
C BY ITERATIVE PROCESS UNTIL REACHING STEADY CONFIGURATIONS
C
C DATE: MARCH, 22TH, 2013
C
      program dorsal
C
C INCLUSION OF PARAMETER AND COMMON FILES. THESE FILES ARE
C GIVEN SEPARATELY UNDER THE FILE NAMES : 'PARAMETER' AND 'COMMON'
C
      include 'parameter'
      include 'common'
C
C EACH RUN (OR CASE) MAY START FROM LAMINAR - HALF SPACE CONDUCTIVE
C CONDITIONS (NEW RUN) ISTART=0; OR MAY RESTART FROM AN INTERMEDIATE
C RESULT(FILE FORT.XXX, XXX=ISTART)
C
C CHOICE OF STARTING CONDITIONS OF THE RUN
C
      print*,'enter the initial condition of the run'
      read*,istart
      if(istart.eq.0)then
      print*,"run starts from laminar-half space conductive conditions"
                     else
      print*,"computation restarts from intermediate file fort.",istart
                     endif
      if(istart.ne.0)goto77
C
C NEW CASE : INPUT OF PHYSICAL PARAMETERS (SEE FILE: INPUT_FILE) AND
C COMPUTATIONS OPTIONS
C
      call data_input
      call computing_options(1)
C
C DEFINITION OF THE COMPUTING GRID INCLUDING THE SILLS AND LENS
C GEOMETRY DEFINED BY READING DATA_FILE IN SUBROUTINE DATA_INPUT
C
      call computation_grid
C
C BOUNDARY AND INITIAL CONDITIONS
C
      call bc_stream_function
      call bc_vorticity
      call bic_temperature
      call viscosity
      call laminar_flow
C
C END OF RUN INITIALIZATION
C FOLLOWED BY ITERATIVE MOTION TEMPERATURE PROCESS (FLAG 78)
C OR BY COMPUTATION RESTARTING FROM AN INTERMEDIATE RESULT (FLAG 77)
C
      go to 78
   77 continue
C
C RESTART OF COMPUTATION FROM AN INTERMEDIARY RESULT
C THE CALLS TO SUBROUTINES DATA_INPUT, COMPUTING_OPTIONS AND
C COMPUTATION_GRID OPEN THE POSSIBILITY TO RESTART THE COMPUTATION
C CHANGING OR NOT CERTAINS PARAMETERS OF THESE INPUT FILES
C
      call data_input
      call computing_options(1)
      call computation_grid
C
C READING THE INTERMEDIATE RESULT FILE
C FINAL RESULTS OF THE RUN WILL BE SAVED ON FILE FORT.ISTART+1
C
      call reading_results(istart)
      inext=istart+1
C
C STARTING THE ITERATIVE MOTION TEMPERATURE PROCESS
C
   78 continue
      do 1 it = 1, itera
      print*
      print*,'temperature motion iterative process iteration:',it
      print*

C
C RECORDS OF TEMPERATURE FOR GLOBAL CONVERGENCE ASSESSMENT
C
      do ix=1,nx
      do iy=1,ny
      teppp(ix,iy)=tep(ix,iy)
      enddo
      enddo
C
C MOTION CALCULATION (STREAM-FUNCTION AND VORTICITY)
C THE CALL TO SUBROUTINE COMPUTING_OPTIONS OPENS THE POSSIBILITY OF
C ADJUSTING THE PARAMETER INTERACTIVELY DURING THE RUN COMPUTATION
C (FLAGS FOR PRINTING INTERMEDIATE RESULTS, AJUSTING TIME-STEPPING, ...)
C
      call computing_options(0)
      call viscosity
      call motion
C
C TEMPERATURE CALCULATION (STREAM-FUNCTION AND VORTICITY)
C
      call conductivity
      call temperature
C
C TEST OF GLOBAL CONVERGENCE
C SAVING INTERMEDIARY RESULTS ON FILE FORT.99 OPENS THE POSSIBILITY
C OF COMPUTATION RESTART IN CASE OF UNWANTED RUN INTERRUPTION
C
      call convergence_xy(it,ip,tep,teppp,cglobal,'mouv-temp.',iconv)
      if(iconv.eq.1)then
                    go to 98
                    else
                    call writing_results(99)
                    endif
    1 continue
   98 continue
C
C SAVING RUN RESULTS
C
      call writing_results(inext)
C
C POST-PROCESSING OF RUN RESULTS
C
C EXTRACTION OF DATA FOR DRAWING
C THIS SUBROUTINE IS WRITEN FOR NOESYS-TRANSFORM GRAPHIC LIBRARY
C NEEDS TO BE ADAPTATED FOR OTHER GRAPHIC LIBRARY
C
      print*
      print*,'extraction of temperature-motion results for graphics'
      print*
      call drawtab(200,x,y,psi,'psi')
C
C KELVIN CELSIUS CONVERSION
C
      do ix=1,nx
      do iy=1,ny
      tepc(ix,iy)=tep(ix,iy)-273.
      enddo
      enddo
C
      call drawtab(201,x,y,tepc,'tep')
      call drawtab(202,x,y,vx,'vx')
      call drawtab(203,x,y,vy,'vy')
      call drawtab(204,x,y,vis,'vis')
      call drawtab(205,x,y,vor,'vor')
C
C  CALCULATION OF TRACERS, THERMAL HISTORY AND COOLING RATES FROM
C  LEVEL IY=2 TO NY-1
C  COOLING RATE WHICH ARE NOT FULLY INCLUDED IN THE OPENING CLOSURE
C  TEMPERATURE RANGE ARE ARBITRARILY SET TO ZERO
C  COOLING RATES RESUTS ARE WRITTEN ON FILES FORT.206 (ICR) and
C  FORT.207 (SCR)
C
C  CALCULATION OF TRACERS TRAJECTORIES
C
      print*
      print*,'calculation of tracers trajectories'
      print*
      call tracer_trajectories
C
C  CALCULATION OF COOLING RATES
C
      print*
      print*,'calculation of cooling rates'
      print*
      call cooling_rates_ICR
      call cooling_rates_SCR
C
C  CALCULATION AND DRAWING OF THERMAL HISTORY ALONG TRACER TRAJECTORIES
C
      print*,
      print*,'calculation of thermal histories'
      print*
      call temperature_history
C
      print*
      print*,'end of run'
      pause 'stop'
      stop
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE OPERATEURD2_XY
C
C RESOLUTION OF
C A D2F/DX2 + B D2F/DY2 + C DF/DX + D DF/DY = E DF/DT + SM
C
C----------------------------------------------------------------------
      subroutine operateurd2_xy(dt,dx,dy,it,f,a,b,c,d,e,sm,
     *                          nsx,idsx,ifsx,nsy,idsy,ifsy)
C
      include 'parameter'
      dimension ax(nx,ny),bx(nx,ny),cx(nx,ny),dpx(nx,ny),
     *          ay(nx,ny),by(nx,ny),cy(nx,ny),dpy(nx,ny),
     *          px(nx,ny),py(nx,ny)
      dimension nsx(ny),idsx(ny,5),ifsx(ny,5),
     *          nsy(nx),idsy(nx,5),ifsy(nx,5)
      dimension a(nx,ny),b(nx,ny),c(nx,ny),d(nx,ny),e(nx,ny),sm(nx,ny)
      dimension f(nx,ny)
C
C     CONSTANTES OF TRIDIAGONAL SYSTEM FOR LAPLACIAN OPERATOR
C
      dx2=dx*dx
      dy2=dy*dy
      do ix=1,nx
          do iy=1,ny
          ax(ix,iy)=a(ix,iy)*dt/dx2-c(ix,iy)*dt/2./dx
          bx(ix,iy)=-2.*dt*a(ix,iy)/dx2-e(ix,iy)
          cx(ix,iy)=a(ix,iy)*dt/dx2+c(ix,iy)*dt/2./dx

          ay(ix,iy)=b(ix,iy)*dt/dy2-d(ix,iy)*dt/2./dy
          by(ix,iy)=-2.*dt*b(ix,iy)/dy2-e(ix,iy)
          cy(ix,iy)=b(ix,iy)*dt/dy2+d(ix,iy)*dt/2./dy
          enddo
      enddo
C
C   PIVOTS X DIRECTION
C
      do iy=1,ny
      nseg=nsx(iy)
      if(nseg.eq.0) goto99
C
          do is=1,nseg
          id=idsx(iy,is)
          if=ifsx(iy,is)
          px(id+1,iy)=bx(id+1,iy)/cx(id+1,iy)
C
              do ix=id+2,if-1
              px(ix,iy)=(bx(ix,iy)-ax(ix,iy)/px(ix-1,iy))/cx(ix,iy)
              enddo
C
          enddo
C
   99 continue
      enddo
C
C   PIVOTS Y DIRECTION
C
      do ix=1,nx
      nseg=nsy(ix)
      if(nseg.eq.0) goto98
C
          do is=1,nseg
	  id=idsy(ix,is)
	  if=ifsy(ix,is)
          py(ix,id+1)=by(ix,id+1)/cy(ix,id+1)
C
              do iy=id+2,if-1
              py(ix,iy)=(by(ix,iy)-ay(ix,iy)/py(ix,iy-1))/cy(ix,iy)
              enddo
C
          enddo
C
   98 continue
      enddo
C
C   ITERATIVE RESOLUTION OF THE OPERATOR
C
      do 1 i=1,it
C
C   X DIRECTION
C
C   CALCULATION OF EQUATION RHS
C
      do ix=1,nx
C
          do iy=1,ny
          dpx(ix,iy)=-2.*e(ix,iy)*f(ix,iy)+sm(ix,iy)*dt
     *               -f(ix,iy-1)*ay(ix,iy)
     *               -f(ix,iy+1)*cy(ix,iy)
     *               -f(ix,iy)*by(ix,iy)
          enddo
C
      enddo
C
C   TRANSFER OF INTERNAL AND EXTERNAL BOUNDARY CONDITIONS IN RHS
C   AND PIVOT ADAPTATION
C
      do iy=1,ny
      nseg=nsx(iy)
      if(nseg.eq.0) goto97

          do is=1,nseg
          id=idsx(iy,is)
	  if=ifsx(iy,is)
      dpx(id+1,iy)=(dpx(id+1,iy)-ax(id+1,iy)*f(id,iy))/cx(id+1,iy)
      dpx(if-1,iy)=dpx(if-1,iy)-cx(if-1,iy)*f(if,iy)
C
		do ix=id+2,if-1
		dpx(ix,iy)=
     *  (dpx(ix,iy)-ax(ix,iy)*dpx(ix-1,iy)/px(ix-1,iy))/cx(ix,iy)
		enddo
C
C   INVERSION OF LAST UNKNOWN VALUE AND PROPAGATION OF SOLUTION
C
                f(if-1,iy)=dpx(if-1,iy)/px(if-1,iy)
C
		do ix=if-2,id+1,-1
		f(ix,iy)=(dpx(ix,iy)-f(ix+1,iy))/px(ix,iy)
		enddo
C
          enddo
C
   97 continue
      enddo
C
C   Y DIRECTION
C
C   CALCULATION OF RHS
C
      do iy=1,ny
C
          do ix=1,nx
          dpy(ix,iy)=-2.*e(ix,iy)*f(ix,iy)+sm(ix,iy)*dt
     *               -f(ix-1,iy)*ax(ix,iy)
     *               -f(ix+1,iy)*cx(ix,iy)
     *               -f(ix,iy)*bx(ix,iy)
          enddo
C
      enddo
C
C   TRANSFER OF INTERNAL AND EXTERNAL BOUNDARY CONDITIONS IN RHS
C   AND PIVOT ADAPTATION
C
      do ix=1,nx
      nseg=nsy(ix)
      if(nseg.eq.0) goto96
C
          do is=1,nseg
          id=idsy(ix,is)
          if=ifsy(ix,is)
      dpy(ix,id+1)=(dpy(ix,id+1)-ay(ix,id+1)*f(ix,id))/cy(ix,id+1)
      dpy(ix,if-1)=dpy(ix,if-1)-cy(ix,if-1)*f(ix,if)
C
              do iy=id+2,if-1
              dpy(ix,iy)=
     *       (dpy(ix,iy)-ay(ix,iy)*dpy(ix,iy-1)/py(ix,iy-1))/cy(ix,iy)
              enddo
C
C   INVERSION OF LAST UNKNOWN VALUE AND PROPAGATION OF SOLUTION
C
              f(ix,if-1)=dpy(ix,if-1)/py(ix,if-1)
              do iy=if-2,id+1,-1
              f(ix,iy)=(dpy(ix,iy)-f(ix,iy+1))/py(ix,iy)
              enddo
C
          enddo
C
   96 continue
      enddo

    1 continue
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE DATA_INPUT
C
C     INPUT OF PHYSICAL PARAMETERS
C
C-----------------------------------------------------------------------
      subroutine data_input
C
      include 'parameter'
      include 'common'
      open(1, file='data_file', form='formatted')
C
      read(1,*)svl
      print*,'left spreading velocity (m/year)         :',svl
      svl=svl/365.25/3600./24.
      print*,'left spreading velocity (m/s)            :',svl
C
      read(1,*)svr
      print*,'right spreading velocity (m/year)        :',svr
      svr=svr/365.25/3600./24.
      print*,'right spreading velocity (m/s)           :',svr
C
      read(1,*)hl,ril
      print*,'height of lens above MTZ                 :',hl
      print*,'rate of intrusion lens(0->1)             :',ril
C
      read(1,*)nsl
      print*,'number of sills on left side :',nsl
      do i=1,nsl
      read(1,*)hls(i), rils(i)
      print*,'left sill number:',i,' height:',hls(i),'rate:',rils(i)
      enddo
c
      read(1,*)nsr
      print*,'number of sills on right side :',nsr
      do i=1,nsr
      read(1,*)hrs(i), rirs(i)
      print*,'right sill number:',i,' height:',hrs(i),'rate :',rirs(i)
      enddo
c
      read(1,*)xh
      print*,'height of computation box (m)                 :',xh
c
      read(1,*)xl
      print*,'half length of computation box(m) (-xl,xl)    :',xl
c
      read(1,*)cdvis
      print*,'cold dynamic viscosity (pa.s)                 :',cdvis
c
      read(1,*)hdvis
      print*,'hot dynamic viscosity (pa.s)                  :',hdvis
c
      read(1,*)ttvis
      print*,'temperature transition of viscosity           :',ttvis
c
      read(1,*)dtvis
      print*,'delta transition viscosity                    :',dtvis
c
      read(1,*)tmi
      print*,'temperature of melt intrusion      (K)        :',tmi
c
      read(1,*)tsurf
      print*,'surface temperature       (K)                 :',tsurf
c
      read(1,*)xkhc
      print*,'thermal conductivity higher crust (J/(K m s)) :',xkhc
c
      read(1,*)xklc
      print*,'thermal conductivity lower crust (J/(K m s))  :',xklc
c
      read(1,*)tcrac
      print*,'cracking temperature conductivity  (K)        :',tcrac
c
      read(1,*)dtcrac
      print*,'delta cracking temperature conductivity (K)   :',dtcrac
c
      read(1,*)ycond
      print*,'height conductivity transition  (m)           :',ycond
c
      read(1,*)dycond
      print*,'delta height conductivity transition (m)      :',dycond
c
      read(1,*)rho
      print*,'reference density (kg/m3)                     :',rho
c
      read(1,*)cp
      print*,'specific heat J/(kg K)                        :',cp
c
      read(1,*)ql
      print*,'latent heat of crystalization (J/kg)          :',ql
c
      read(1,*)toicr,tcicr
      print*,'opening-closure temp. igneous Cooling R.  :',toicr,tcicr
c
      read(1,*)toscr,tcscr
      print*,'opening-closure temp. subsolidus CoolingR:',toscr,tcscr
c
      read(1,*)g
      print*,'gravity acceleration                          :',g
      print*


      close(1)
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE COMPUTATION_GRID
C
C     CALCULATION OF THE COMPUTATION GRID
C
C-----------------------------------------------------------------------
      subroutine computation_grid
C
      include 'parameter'
      include 'common'
C
C   DX   : GRID STEP IN THE X DIRECTION (HORIZONTAL)
C   DY   : GRID STEP IN THE Y DIRECTION (VERTICAL)
C   X(IX): FROM X(1)=-XL/2 TO X(NX)=XL/2
C   Y(IY): FROM Y(1)=0 TO Y(NY)=XH
C
      dx=xl/float(nxm)
      dy=xh/float(nym)
      ddx=2.*dx
      ddy=2.*dy
      dx2=dx*dx
      dy2=dy*dy
      qdx2=4.*dx2
      qdy2=4.*dy2
      qdxdy=4.*dx*dy
C
      do ix=1,nx
      x(ix)=-xl/2.+dx*float(ix-1)
      enddo
C
      do iy=1,ny
      y(iy)=dy*float(iy-1)
      prof(iy)=xh-y(iy)
      enddo
C
C     IYL SHALLOW LENS POSITION
C     NSL NUMBER OF LEFT SILLS - IYSL POSITIONS OF THE SILLS
C     NSR NUMBER OF RIGHT SILLS - IYSR POSITIONS OF THE SILLS
C
      iyl=hl/xh*ny
      iylp=iyl+1
      iylm=iyl-1

      if(nsl.ne.0)then
                  do i=1,nsl
                  iysl(i)=hls(i)/xh*ny
                  enddo
                  endif
      if(nsr.ne.0)then
                  do i=1,nsr
                  iysr(i)=hrs(i)/xh*ny
                  enddo
                  endif
C
C   HORIZONTAL SEGMENTATION OF THE GRID FOR MID RIDGE INTERNAL CONDITIONS
C   NSXM(IY):NUMBER OF SEGMENTS AT HORIZONTAL LEVEL IY (5 MAXIMUM)
C   IBSXM(IY,IS) : IX-INDICE BEGINNING OF SEGMENT IS AT LEVEL IY
C   IESXM(IY,IS) : IX-INDICE ENDING OF SEGMENT IS AT LEVEL IY
C   INITIALIZATION : THE NUMBER OF SEGMENT IS SET TO 1 (GENERAL CASE)
C
                   	do iy=1,ny
			nsxm(iy)=1
			ibsxm(iy,1)=1
			iesxm(iy,1)=nx
			enddo
			nsxm(1)=0
			nsxm(ny)=0
C
C   VERTICAL SEGMENTATION OF THE GRID FOR SHEETED DYKE LAYER MODELING
C   NSYM(IX):NUMBER OF SEGMENTS AT VERTICAL INDICE IX (5 MAXIMUM)
C   IBSYM(IX,IS) : IX-INDICE BEGINNING OF SEGMENT IS AT COLUMN IX
C   IESXM(IX,IS) : IX-INDICE ENDING OF SEGMENT IS AT COLUMN IX
C   INITIALIZATION :THE NUMBER OF SEGMENT IS SET TO 1 (GENERAL CASE)
C
C FOR MOTION (STREAM FUNCTION AND VORTICITY RESOLUTIONS)
C
			do ix=1,nx
			nsym(ix)=1
			ibsym(ix,1)=1
			iesym(ix,1)=ny
			enddo
			nsym(1)=0
			nsym(nx)=0
C
C FOR TEMPERATURE RESOLUTION
C
			do iy=1,ny
			nsxt(iy)=1
			ibsxt(iy,1)=1
			iesxt(iy,1)=nx
			enddo
			nsxt(1)=0
			nsxt(ny)=0


			do ix=1,nx
			nsyt(ix)=1
			ibsyt(ix,1)=1
			iesyt(ix,1)=ny
			enddo
			nsyt(1)=0
			nsyt(nx)=0

C
C SPLITTING OF ROWS FROM MTZ LEVEL TO THE SHALLOW LENS LEVEL
C AT NXS2 AND NXS2+1 TO TAKE INTO ACCOUNT THE
C PRESENCE OF INTRUSIVE SILLS AT THE MID-RIDGE LOCATION
C
C MOTION RESOLUTION
C
			do iy=2,iylm
			nsxm(iy)=2
			ibsxm(iy,1)=1
			iesxm(iy,1)=nxs2
                        ibsxm(iy,2)=nxs2p
			iesxm(iy,2)=nx
			enddo
			nsxm(1)=0
			do iy=iyl,ny
			nsxm(iy)=0
			enddo
C
C TEMPERATURE RESOLUTION
C
			do iy=2,nym
			nsxt(iy)=2
			ibsxt(iy,1)=1
			iesxt(iy,1)=nxs2
                        ibsxt(iy,2)=nxs2p
			iesxt(iy,2)=nx
			enddo
			nsxt(1)=0
			nsxt(ny)=0
C
C MOTION RESOLUTION
C
			do ix=1,nx
			nsym(ix)=1
			ibsym(ix,1)=1
			iesym(ix,1)=iylp
			enddo
			nsym(1)=0
			nsym(nxs2)=0
			nsym(nxs2p)=0
			nsym(nx)=0
C
C TEMPERATURE RESOLUTION
C
			do ix=1,nx
			nsyt(ix)=1
			ibsyt(ix,1)=1
			iesyt(ix,1)=ny
			enddo
			nsyt(1)=0
			nsyt(nxs2)=0
			nsyt(nxs2p)=0
			nsyt(nx)=0
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE BC_STREAM_FUNCTION
C
C     SETS THE STREAM FUNCTION INTERNAL AND EXTERNAL BOUNDARY CONDITIONS
C     AS A FUNCTION OF LEFT AND RIGHT SPREADING VELOCITIES AND LENS AND
C     SILLS MELT INTRUSION AT THE RIDGE
C
C-----------------------------------------------------------------------
      subroutine bc_stream_function
C
      include 'parameter'
      include 'common'
C
C MATHEMATICALLY THE STREAM FUNCTIO IS DEFINED TO AN ARBITRARY CONSTANT,
C WHICH WILL BE CHOSEN TO GET A ZERO STREAM FUNCTION AT THE UPPER
C IN CASE OF SYMETRIC LEFT AND RIGHT SPREADING.
C
C THE FLUX OF INTRUDING MELT ENTERING THE COMPUTATION BOX AT THE RIDGE,
C PSIRIDGE, BALANCES THE FLUX OF MATTER THROUGH THE LEFT AND RIGHT
C LATERAL BOUNDARIES
C
C THE FLUX OF MELT CROSSING THE LENS ROOF TO BUILT THE SHEETED DYKES,
C PSIDYKE, BALANCES THE FLUX OF MATTER CORRESPONDING TO THE SHEETED
C DYKE THROUGH THE LEFT AND RIGHT LATERAL BOUNDARIES
C
C PSIGABBRO CORRESPONDS TO THE REMAINING AVAILABLE TO BUILT THE LOWER
C CRUST
C
      psiridge=(svr+svl)*xh
      psidyke=(svr+svl)*(xh-y(iyl))
      psigabbro=psiridge-psidyke
C
C STREAM FUNCTION AND VELOCITY BOUNDARY CONDITIONS ON THE BOTTOM LEFT
C BOUNDARY OF THE COMPUTATION BOX
C
      do ix=1,nxs2
      psi(ix,1)=0.+psiridge/2.
      vx(ix,ny)=-svl
      vy(ix,ny)=0.
      enddo
C
C TOP LEFT
C
      do ix=1,nxs2
      psi(ix,ny)=-svl*xh+psiridge/2.
      vx(ix,ny)=-svl
      vy(ix,ny)=0.
      enddo
C
C TOP RIGHT
C
      do ix=nxs2p,nx
      psi(ix,ny)=-svl*xh+psiridge/2.
      vx(ix,ny)=svr
      vy(ix,ny)=0.
      enddo
C
C BOTTOM RIGHT
C
      do ix=nxs2p,nx
      psi(ix,1)=-(svr+svl)*xh+psiridge/2.
      vx(ix,ny)=svr
      vy(ix,ny)=0.
      enddo
C
C LEFT AND RIGHT LATERAL BOUNDARIES
C
      do iy=1,ny
      psi(1,iy)=psi(1,1)-svl*y(iy)
      vx(1,iy)=-svl
      vy(1,iy)=0.
      psi(nx,iy)=psi(nx,1)+y(iy)*svr
      vx(nx,iy)=svr
      vy(nx,iy)=0.
      enddo
C
C INTERNAL BOUNDARY CONDITIONS AT THE RIDGE TAKING INTO ACCOUNT
C THE SILLS AND LENS STRUCTURE FOR MELT INTRUSION
C
C LEFT SILLS
C
C NO SILL
C
      if(nsl.eq.0)	then
      			do iy=2,iylm
      			psi(nxs2,iy)=psi(nxs2,1)
      			enddo
      			goto99
			endif
C
C NSL LEFT SILLS
C
      id=2
      tis=0.
      do i=1,nsl
		if=iysl(i)
			do iy=id,if
			psi(nxs2,iy)=psi(nxs2,id-1)-psigabbro*tis
			enddo
		id=if+1
		tis=rils(i)
      enddo
		if=iylm
			do iy=id,if
			psi(nxs2,iy)=psi(nxs2,id-1)-psigabbro*tis
			enddo
   99 continue
C
C SHALLOW LENS AND SHEETED DYKE LAYER MELT INTRUSION
C
      psi(nxs2,iyl)=psi(nxs2,iylm)-psigabbro*ril/2.
      psi(nxs2p,iyl)=psi(nxs2,iyl)-psidyke
      psi(nxs2p,iylm)=psi(nxs2p,iyl)-psigabbro*ril/2.
C
C RIGHT SILLS
C
C NO SILL
C
      if(nsr.eq.0)  then
					do iy=iylm-1,2,-1
					psi(nxs2p,iy)=psi(nxs2p,iylm)
					enddo
					goto98
					endif
C
C NSR RIGHT SILLS
C
      id=iylm-1
      tis=0.
		do i=1,nsr
		if=(hrs(nsr+1-i)/xh)*ny+1
			do iy=id,if,-1
			psi(nxs2p,iy)=psi(nxs2p,id+1)-psigabbro*tis
			enddo
		id=if-1
		tis=rirs(nsr+1-i)
		enddo
		if=2
			do iy=id,if,-1
			psi(nxs2p,iy)=psi(nxs2p,id+1)-psigabbro*tis
			enddo
   98 continue
C
C SHEETED DYKES ABOVE THE SHALLOW LENS
C
C LEFT AND RIGHT PARTS
C
	  do iy=iyl,nym
C
		do ix=2,nxs2
		psi(ix,iy)=psi(1,iy)
		vx(ix,iy)=-svl
                vy(ix,iy)=0.
		enddo
C
		do ix=nxs2p,nxm
		psi(ix,iy)=psi(nx,iy)
		vx(ix,iy)=svr
                vy(ix,iy)=0.
		enddo

      enddo
C
C VELOCITY AT THE RIDGE AXIS (TWO CENTRAL COLUMNS OF THE GRID)
C
      ddy=2.*dy
C
      do ix=nxs2,nxs2p
         do iy=2,nym
         vx(ix,iy)=(psi(ix,iy+1)-psi(ix,iy-1))/ddy
         enddo
      enddo
C
      ddx=2.*dx
C
      do ix=nxs2,nxs2p
         do iy=2,nym
         vy(ix,iy)=-(psi(ix+1,iy)-psi(ix-1,iy))/ddx
         enddo
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE BC_VORTICITY
C
C     SETS THE VORTICITY INTERNAL AND EXTERNAL BOUNDARY CONDITIONS
C     (FREE SIP)
C
C-----------------------------------------------------------------------
      subroutine bc_vorticity
C
      include 'parameter'
      include 'common'
C
      do ix=1,nx
      do iy=1,ny
      vor(ix,iy)=0.
      enddo
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE BIC_TEMPERATURE
C
C     SETS THE BOUNDARY CONDTIONS AND THE INITIAL CONDITIONS OF THE
C     TEMPERATURE
C
C-----------------------------------------------------------------------
      subroutine bic_temperature
C
      include 'parameter'
      include 'common'
C
C INITIALISATION OF TEMPERATURE TO INFINITE HALF SPACE COOLING VALUES
C
C LEFT SIDE
C
      print*,
      print*,'subroutine bic_temperature: initializing temperature'
C
      dif=xklc/rho/cp
      vitg=dabs(svl)

	  do ix=1,nxs2
	  xx=dabs(x(ix))
	  do iy=1,ny
	  x1=prof(iy)/2.*dsqrt(vitg/dif/xx)
          call phm_function_erf(x1,x2)
          tep(ix,iy)=(tmi-tsurf)*x2+tsurf
C         print*,'half pace cooling temperature',ix,iy,tep(ix,iy)
       	  enddo
       	  enddo
C
C RIGHT SIDE
C
      vitd=dabs(svr)

      do ix=nxs2p,nx
	  xx=dabs(x(ix))
	  do iy=1,ny
	  x1=prof(iy)/2.*dsqrt(vitd/dif/xx)
	  call phm_function_erf(x1,x2)
          tep(ix,iy)=(tmi-tsurf)*x2+tsurf
C         print*,'half pace cooling temperature',ix,iy,tep(ix,iy)
	  enddo
	  enddo
C
C TEMPERATURE BOUNDARY CONDITIONS AT THE RIDGE AXIS
C
      do iy=1,iyl+1
      tep(nxs2,iy)=tmi
      tep(nxs2p,iy)=tmi
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE CONDUCTIVITY
C
C     CALCULATES THE CONDUCTIVITY ACCORDING TO TEMPERATURE AND DEPTH
C
C-----------------------------------------------------------------------
      subroutine conductivity
C
      include 'parameter'
      include 'common'
C
C   GAMMA STEP FUNCTION - CRACKING TEMPERATURE
C   GAMH=1/2 ( 1 + TANH( (T-TC)/DTC ) )
C
      do ix=1,nx
C
          do iy=1,ny
          x1=(tep(ix,iy)-tcrac)/dtcrac
          gamh(ix,iy)=.5*(1.+tanh(x1))
          dgamh(ix,iy)=2.*gamh(ix,iy)*(1.-gamh(ix,iy))
          enddo
C
      enddo
C
C   GAMMA STEP FUNCTION - DEPTH
C
      do iy=1,ny
      x1=(y(iy)-ycond)/dycond
      gamc(iy)=.5*(1.+tanh(x1))
      enddo
C
C   CONDUCTIVITY
C
      do ix=1,nx
C
          do iy=1,ny
      xk(ix,iy)=xklc+(xklc*gamh(ix,iy)+xkhc*(1.-gamh(ix,iy)))*gamc(iy)
          enddo
C
      enddo
C
      call dtabdx(xk,dxkdx)
      call dtabdy(xk,dxkdy)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE VISCOSITY
C
C     CALCULATES THE VISCOSITY DEPENDING ON TEMPERATURE
C
C-----------------------------------------------------------------------
      subroutine viscosity
C
      include 'parameter'
      include 'common'
C
C  CALCULATION OF THE GAMMA STEP FUNCTION
C  GAMV=1/2 ( 1 + TANH( (T-TC)/DTC ) )
C
      do ix=1,nx
      do iy=1,ny
      x1=(tep(ix,iy)-ttvis)/dtvis
      gamv(ix,iy)=.5*(1.+tanh(x1))
      dgamvdx(ix,iy)=2.*gamv(ix,iy)*(1.-gamv(ix,iy))
      enddo
      enddo
C
C  CALCULATION OF VISCOSITY AND DERIVATIVES
C  GAMV=1/2 ( 1 + TANH( (T-TC)/DTC ) )
C
      do ix=1,nx
      do iy=1,ny
      vis(ix,iy)=hdvis*gamv(ix,iy)+cdvis*(1.-gamv(ix,iy))
      enddo
      enddo
C
      call dtabdx(vis,dvdx)
      call dtabdx(dvdx,d2vdx2)
      call dtabdy(vis,dvdy)
      call dtabdy(dvdy,d2vdy2)
      call dtabdx(dvdy,z11)
      call dtabdy(dvdx,z12)
C
      do ix=1,nx
      do iy=1,ny
      d2vdxdy(ix,iy)=.5*(z11(ix,iy)+z12(ix,iy))
      enddo
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE LAMINAR_FLOW
C
C     CALCULATION OF THE LAMINAR FLOW INDUCED BY BOUNDARY CONDITIONS,
C     INITIAL TEMPERATURE AND VISCOSITY DEPENDENCE ON TEMPERATURE
C     THE LAMINAR FLOW WILL SERVE AS INITIALIZATION FOR ITERATIVE MOTION
C     CALCULATION
C
C-----------------------------------------------------------------------
      subroutine laminar_flow
C
      include 'parameter'
      include 'common'
C
C  ITERATIVE STREAM FUNCTION-VORTICITY RESOLUTIONS TO TAKE VISCOSITY
C  VARIATIONS INTO ACCOUNT
C
      print*
      print*,'subroutine laminar_flow: initializing stream-function'
C
      do it=1,100
C
      call computing_options(0)
      call stream_function
      call vorticity
C
      call convergence_xy(it,ipm,vit,vitp,cvit,'motion',iconv)
      if(iconv.eq.1)goto99
C
      enddo
   99 continue
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE DTABDY
C
C     CALCULATION OF FIRST DERIVATIVE OF ARRAY DT(X,Y)/DY
C
C----------------------------------------------------------------------
      subroutine dtabdy(t,dt)
C
      include 'parameter'
      common/igrilles/iyl,iysr(ny),iysl(ny)
      common/xgrilles/x(nx),dx,ddx,dx2,qdx2,
     *                y(ny),dy,ddy,dy2,qdy2,prof(ny)
      dimension t(nx,ny), dt(nx,ny)
C
      ddy=2.*dy
C
      do ix=1,nx
C
		do iy=2,nym
		dt(ix,iy)=(t(ix,iy+1)-t(ix,iy-1))/ddy
		enddo
C
		dt(ix,1)=(-3.*t(ix,1)+4.*t(ix,2)-t(ix,3))/ddy
		dt(ix,ny)=(3.*t(ix,ny)-4.*t(ix,ny-1)+t(ix,ny-2))/ddy
C
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE DTABDX
C
C     CALCULATION OF FIRST DERIVATIVE OF ARRAY DT(X,Y)/DX
C
C----------------------------------------------------------------------
      subroutine dtabdx(t,dt)
C
      include 'parameter'
      common/igrilles/iyl,iysr(ny),iysl(ny)
      common/xgrilles/x(nx),dx,ddx,dx2,qdx2,
     *              y(ny),dy,ddy,dy2,qdy2,prof(ny)
      dimension t(nx,ny),dt(nx,ny)
C
      ddx=2.*dx
C
      do iy=1,ny
C
		do ix=2,nxm
		dt(ix,iy)=(t(ix+1,iy)-t(ix-1,iy))/ddx
		enddo
C
      dt(1,iy)=(-3.*t(1,iy)+4.*t(2,iy)-t(3,iy))/ddx
      dt(nx,iy)=(3.*t(nx,iy)-4.*t(nx-1,iy)+t(nx-2,iy))/ddx
C
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE PHM_FUNCTION_ERF
C
C     CALCULATION OF ERROR FUNCTION
C     ERF(X)=2./SQRT(PI) INTEGRAL DE 0 TO X (EXP (-T**2) DT)
C     VERSION 1.0 29 NOVEMBRE 2005
C----------------------------------------------------------------------
      subroutine phm_function_erf(x1,x2)
      implicit doubleprecision (a-h,o-z)

      dimension f(20000)
      n=20000
      pi=dacos(-1.d0)
      xd=0.d0
      xf=x1
      dx=x1/float(n-1)
      do i=1,n
      x=float(i-1)*dx
      f(i)=dexp(-x*x)
      enddo
      call phm_integral_trapeze_function_equi(xd,xf,f,n,x2)
      x2=2.d0*x2/dsqrt(pi)
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE PHM_INTEGRAL_TRAPEZE_FUNCTION_EQUI
C
C     CALCULATION OF THE INTEGRAL OF CONTINUOUS FUNCTION FROM X1 TO X2
C     VALUES OF THE FUNCTION ARE GIVEN IN ARRAY F OF DIMENSION N
C     N MUST BE GREATER THAN 2.
C     VERSION 1.0 29 NOVEMBRE 2005
C----------------------------------------------------------------------
      subroutine phm_integral_trapeze_function_equi(x1,x2,f,n,xint)
C
      implicit doubleprecision (a-h,o-z)
      dimension f(n)
C
      xint=0.5*(f(1)+f(n))
      dx=(x2-x1)/float(n-1)
C
      do i=2,n-1
      xint=xint+f(i)
      enddo
C
      xint=xint*dx
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE interpolation4p_y
C
C----------------------------------------------------------------------
      subroutine interpolation4p_y(tx,ty,txi,tyi)

      include 'parameter'
      dimension tx(ny),ty(ny)
      dimension ind(4)
      dimension xt(4),yt(4)
C
      nyi=4
C
      do iy=1,nym
C
      if(
     &   (txi.ge.tx(iy)).and.(txi.lt.tx(iy+1))
     &                  .or.
     &   (txi.le.tx(iy)).and.(txi.gt.tx(iy+1))
     &   )
     &			then
                        iyd=iy
                        go to 99
      			endif
C
      enddo
   99 continue
C
      call indices_y(iyd,nyi,ind)
C
      do iy=1,nyi
      xt(iy)=tx(ind(iy))
      yt(iy)=ty(ind(iy))
      enddo
C
      call polint(xt,yt,nyi,txi,tyi,dtyi)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE interpolation4p_x
C
C----------------------------------------------------------------------
      subroutine interpolation4p_x(nxd,nxf,tx,ty,txi,tyi,ipr)
C
      include 'parameter'
      dimension tx(nx),ty(nx)
      dimension ind(4)
      dimension xt(4),yt(4)
C
      nxi=4
      do ix=nxd,nxf
C
      if(
     &   (txi.ge.tx(ix)).and.(txi.lt.tx(ix+1))
     &                  .or.
     &   (txi.le.tx(ix)).and.(txi.gt.tx(ix+1))
     &   )
     &			then
                        ixd=ix
                        go to 99
       			endif
C
      enddo
   99 continue
C
      call indices_x(ixd,nxi,ind)
C
      do i=1,nxi
      xt(i)=tx(ind(i))
      yt(i)=ty(ind(i))
      enddo
C
      call polint(xt,yt,nxi,txi,tyi,dtxi)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE indices_y
C
C----------------------------------------------------------------------
      subroutine indices_y(iyd,nyi,ind)
C
      include 'parameter'
      common/igrilles/iyl,iysr(ny),iysl(ny)
      dimension ind(4)
C
      ind(1)=iyd-nyi/2+1
C
      do iy=2,nyi
      ind(iy)=ind(iy-1)+1
      enddo
C
      if(ind(1).lt.1)then
                     ind(1)=1
                     do iy=2,nyi
                     ind(iy)=ind(iy-1)+1
                     enddo
                     goto99
                     endif
C
      if((ind(nyi).gt.iyl+1).and.(iyd.lt.iyl+1))then
                                                ind(nyi)=iyl+1
                                                do iy=nyi-1,1,-1
                                                ind(iy)=ind(iy+1)-1
                                                enddo
                                                goto99
                                                endif
C
      if((ind(1).lt.iyl+1).and.(iyd.ge.iyl+1))then
                                              ind(1)=iyl+1
                                              do iy=2,nyi
                                              ind(iy)=ind(iy-1)+1
                                              enddo
                                              goto99
                                              endif
C
      if(ind(nyi).gt.ny)then
                        ind(nyi)=ny
                        do iy=nyi-1,1,-1
                        ind(iy)=ind(iy+1)-1
                        enddo
                        goto99
                        endif
C
   99 continue
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE indices_x
C
C----------------------------------------------------------------------
      subroutine indices_x(ixd,nxi,ind)
C
      include 'parameter'
      dimension ind(4)
C
      ind(1)=ixd-nxi/2+1
      do ix=2,nxi
      ind(ix)=ind(ix-1)+1
      enddo
C
      if(ind(1).lt.1)then
                     ind(1)=1
                     do ix=2,nxi
                     ind(ix)=ind(ix-1)+1
                     enddo
                     goto99
                     endif
C
      if((ind(nxi).gt.nxs2).and.(ind(1).lt.nxs2))then
                                                 ind(nxi)=nxs2
                                                 do ix=nxi-1,1,-1
                                                 ind(ix)=ind(ix+1)-1
                                                 enddo
                                                 goto99
                                                 endif
C
      if((ind(1).le.nxs2+1).and.(ind(nxi).gt.nxs2+1))then
                                                     ind(1)=nxs2+1
                                                     do ix=2,nxi
                                                ind(ix)=ind(ix-1)+1
                                                     enddo
                                                     goto99
                                                     endif
C
      if(ind(nxi).gt.nx)then
                        ind(nxi)=nx
                        do ix=nxi-1,1,-1
                        ind(ix)=ind(ix+1)-1
                        enddo
                        goto99
                        endif
C
   99 continue
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE POLINT (NUMERICAL RECIPES P. 82)
C
C----------------------------------------------------------------------
      subroutine polint(xa,ya,n,x,y,dy)
C
C GIVEN ARRAYS XA AND YA, EACH OF LENGTH N, AND GIVEN A VALUE X, THIS
C ROUTINE RETURNS A VALUE Y, AND AN ERROR ESTIMATE DY. IF P(X) IS THE
C POLYNOMIAL OF DEGREE N-1SUCH THAT P(XA(I))=YA(I),I=0,...,N, THEN THE
C RETURNED VALUE Y=P(X). CHANGE NMAX AS DESIRED TO BE THE LARGEST
C ANTICIPATED VALUE OF N
C
      implicit double precision (a-h,o-z)
      parameter (nmax=10)
      dimension xa(n),ya(n),c(nmax),d(nmax)
C
      ns=1
      dif=dabs(x-xa(1))
      do 11 i=1,n
         dift=dabs(x-xa(i))
         if(dift.lt.dif) then
                         ns=i
                         dif=dift
                         endif
         c(i)=ya(i)
         d(i)=ya(i)
   11 continue
      y=ya(ns)
      ns=ns-1
      do 13 m=1,n-1
            do 12 i=1,n-m
            ho=xa(i)-x
            hp=xa(i+m)-x
            w=c(i+1)-d(i)
            den=ho-hp
            if(den.eq.0.)then
                         call polint_2(xa,ya,n,x,y,dy)
                         return
                         endif

            den=w/den
            d(i)=hp*den
            c(i)=ho*den
   12       continue
      if(2*ns.lt.n-m)then
                      dy=c(ns+1)
                      else
                      dy=d(ns)
                      ns=ns-1
                      endif
      y=y+dy
   13 continue
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE POLINT2 (NUMERICAL RECIPES P. 82)
C
C----------------------------------------------------------------------
      subroutine polint_2(xa,ya,n,x,y,dy)
C
      implicit double precision (a-h,o-z)
      dimension xa(n),ya(n)
C
      do i=1,n
           if((x.ge.xa(i)).and.(x.lt.xa(i+1))
     &                    .or.
     &        (x.le.xa(i)).and.(x.gt.xa(i+1)))then
                                              id=i
                                              go to 99
                                              endif
      enddo
   99 continue
      dy=1.d+99
      call interpol_2(xa(i),ya(i),xa(i+1),ya(i+1),x,y)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE INTERPOL_2
C
C----------------------------------------------------------------------
      subroutine interpol_2(x1,y1,x2,y2,x,y)
C
      implicit double precision (a-h,o-z)
C
      y=y1+(y2-y1)*(x-x1)/(x2-x1)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE stream_function
C
C     Resolution of the Stream_function
C
C----------------------------------------------------------------------
      subroutine stream_function
C
      include 'parameter'
      include 'common'
      dimension a(nx,ny),b(nx,ny),c(nx,ny),d(nx,ny),e(nx,ny),sm(nx,ny)
C
C  COEFFICIENTS OF THE ELLIPTIC OPERATOR
C
C         A D2F/DX2 + B D2F/DY2 + C DF/DX + D DF/DY = E DF/DT + SM
C
      do ix=1,nx
      do iy=1,ny
      a(ix,iy)=1.
      b(ix,iy)=1.
      c(ix,iy)=0.
      d(ix,iy)=0.
      e(ix,iy)=1.
      sm(ix,iy)=-vor(ix,iy)
      enddo
      enddo
C
C  NUMBER OF ADI UNDER ITERATIONS AND TIME STEPPING IN SUBROUTINE
C  OPERATEURD2_XY
C
      itlap=10
      dt=cour_psi*dx2
C
C  RECORDS OF PRESENT STREAM FUNCTION FOR CONVERGENCE ASSESSMENT
C
      do 1 it=1,itp

	  do ix=1,nx
	  do iy=1,ny
	  psip(ix,iy)=psi(ix,iy)
	  enddo
	  enddo

      call operateurd2_xy(dt,dx,dy,itlap,psi,a,b,c,d,e,sm,
     *                    nsxm,ibsxm,iesxm,nsym,ibsym,iesym)
C
C  STREAM FUNCTION CONVERGENCE TEST
C
      call convergence_xy
     *       (it,ipp,psi,psip,cpsi,'fonction de courant',iconv)
      if(iconv.eq.1)goto97
C
    1 continue
   97 continue
C
      call velocity
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE VORTICITY
C
C     RESOLUTION OF VORTICITY
C
C----------------------------------------------------------------------
      subroutine vorticity
C
      include 'parameter'
      include 'common'
      dimension a(nx,ny),b(nx,ny),c(nx,ny),d(nx,ny),e(nx,ny),sm(nx,ny)
C
C  COEFFICIENTS OF THE ELLIPTIC OPERATOR
C
C         A D2F/DX2 + B D2F/DY2 + C DF/DX + D DF/DY = E DF/DT + SM
C
      do ix=1,nx
      do iy=1,ny
      a(ix,iy)=1.d0
      b(ix,iy)=1.d0
      c(ix,iy)=0.d0
      d(ix,iy)=0.d0
      e(ix,iy)=1.d0
      enddo
      enddo
C
C  NUMBER OF ADI UNDER ITERATIONS AND TIME STEPPING IN SUBROUTINE
C  OPERATEURD2_XY
C
      itlap=10
      dt=cour_vor*dx2
C
C  CALCUL OF VISCOSITY*VORTICITY
C
      do ix=1,nx
      do iy=1,ny
      vvor(ix,iy)=vis(ix,iy)*vor(ix,iy)
      enddo
      enddo
C
C  CALCUL OF SM =
C        -4 D2P/DXDY D2V/DXDY -2 (D2P/DX2 D2V/DY2 + D2P/DY2 D2V/DX2)
C        -4 D VX/DX D2 V/DXDY +2 (D VY/DX D2V/DY2 - D VX/DY D2V/DX2)
C
      do ix=1,nx
      do iy=1,ny
      sm(ix,iy)=-4.*dvxdx(ix,iy)*d2vdxdy(ix,iy)
     *    +2.* (dvydx(ix,iy)*d2vdy2(ix,iy)-dvxdy(ix,iy)*d2vdx2(ix,iy))
      enddo
      enddo
C
C  RECORDS OF VISCOSITY X VORTICITY FOR CONVERGENCE ASSESSMENT
C
      do 1 it=1,itv
C
      do ix=1,nx
	  do iy=1,ny
	  vvorp(ix,iy)=vvor(ix,iy)
	  enddo
	  enddo
C
      call operateurd2_xy(dt,dx,dy,itlap,vvor,a,b,c,d,e,sm,
     *                    nsxm,ibsxm,iesxm,nsym,ibsym,iesym)
C
C  VISCOSITY X VORTICITY CONVERGENCE TEST
C
      call convergence_xy(it,ipv,vvor,vvorp,cvor,'vorticite',iconv)
      if(iconv.eq.1)goto97
C
    1 continue
   97 continue
C
C  CALCULATION OF VISCOSITY
C
      do ix=1,nx
      do iy=1,ny
      vor(ix,iy)=vvor(ix,iy)/vis(ix,iy)
      enddo
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE MOTION
C
C     ITERATIVE RESOLUTIONS OF STREAM FUNCTION AND VORTICIY
C
C----------------------------------------------------------------------
      subroutine motion
C
      include 'parameter'
      include 'common'
C
      do it=1,iterm
C
      call computing_options(0)
      call stream_function
      call vorticity
      call convergence_xy(it,ipm,vit,vitp,cvit,'motion',iconv)
      if(iconv.eq.1)goto99
C
      enddo
   99 continue
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE TEMPERATURE
C
C     RESOLUTION OF TEMPERATURE
C
C----------------------------------------------------------------------
      subroutine temperature
C
      include 'parameter'
      include 'common'
      dimension a(nx,ny),b(nx,ny),c(nx,ny),d(nx,ny),e(nx,ny),sm(nx,ny)
C
C  COEFFICIENTS OF THE OPERATOR
C
C         A D2F/DX2 + B D2F/DY2 + C DF/DX + D DF/DY = E DF/DT + SM
C
      dt=cour_tep*dx2
      itlap=10

      call viscous_heating
      do 1 it=1,itt
C
      x1=rho*cp
      x2=ql/dtvis
      do ix=1,nx
      do iy=1,ny
      x3=dgamvdx(ix,iy)*x2-x1
      a(ix,iy)=xk(ix,iy)
      b(ix,iy)=xk(ix,iy)
      c(ix,iy)=dxkdx(ix,iy)+x3*vx(ix,iy)
      d(ix,iy)=dxkdy(ix,iy)+x3*vy(ix,iy)
      e(ix,iy)=x1
      sm(ix,iy)=-phi(ix,iy)
      enddo
      enddo
C
C  RECORDS OF TEMPERATURE FOR CONVERGENCE ASSESSMENT
C
       do ix=1,nx
	   do iy=1,ny
	   tepp(ix,iy)=tep(ix,iy)
	   enddo
	   enddo
C
       call operateurd2_xy(dt,dx,dy,itlap,tep,a,b,c,d,e,sm,
     *                     nsxt,ibsxt,iesxt,nsyt,ibsyt,iesyt)
C
C  TEMPERATURE CONVERGENCE TEST
C
      call convergence_xy(it,ipt,tep,tepp,ctep,'temp.',iconv)
      if(iconv.eq.1)goto97
C
    1 continue
   97 continue
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE VISCOUS_HEATING
C
C     CALCULATION OF VISCOUS_HEATING FOR TEMPERATURE RESOLUTION
C
C----------------------------------------------------------------------
      subroutine viscous_heating
C
      include 'parameter'
      include 'common'
C
C    SIGIK = ETA ( D(VI)/DXK + D(VK)/DXI )
C
      do ix=1,nx
C
          do iy=1,ny
          sigxx(ix,iy)=2.*vis(ix,iy)*dvxdx(ix,iy)
          sigxy(ix,iy)=vis(ix,iy)*(dvxdy(ix,iy)+dvydx(ix,iy))
          sigyy(ix,iy)=2.*vis(ix,iy)*dvydy(ix,iy)
          enddo
C
      enddo
C
      do ix=1,nx
C
          do iy=1,ny
          phi(ix,iy)=sigxx(ix,iy)*dvxdx(ix,iy)
     *              +sigxy(ix,iy)*(dvxdy(ix,iy)+dvydx(ix,iy))
     *              +sigyy(ix,iy)*dvydy(ix,iy)
          enddo
C
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE VELOCITY
C
C     CALCUL OF VELOCITY COMPONENTS
C
C----------------------------------------------------------------------
      subroutine velocity
C
      include 'parameter'
      include 'common'
C
C RECARDING OF THE PREVIOUS VALUE OF VELOCITY
C
      do ix=1,nx
      do iy=1,ny
      vitp(ix,iy)=vit(ix,iy)
      enddo
      enddo
C
C CALCULATION OF VX FROM THE NEW STREAM-FUNCTION VALUES
C
      ddy=2.*dy
C
      do ix=1,nx
C
      do iy=2,nym
      vx(ix,iy)=(psi(ix,iy+1)-psi(ix,iy-1))/ddy
      enddo
C
      vx(ix,1)=(-3.*psi(ix,1)+4.*psi(ix,1+1)-psi(ix,1+2))/ddy
      vx(ix,ny)=(3.*psi(ix,ny)-4.*psi(ix,ny-1)+psi(ix,ny-2))/ddy
      enddo
C
C CALCULATION OF VY FROM THE NEW STREAM-FUNCTION VALUES
C
      ddx=2.*dx
C
C BOUNDARY CONDITIONS AT THE RIDGE AXIS
C
      do iy=1,ny
C
      vy(1,iy)=0.
      vy(nxs2,iy)=0.
      vy(nxs2p,iy)=0.
      vy(nx,iy)=0.
C
      do ix=2,nxs2-1
      vy(ix,iy)=(psi(ix-1,iy)-psi(ix+1,iy))/ddx
      enddo
C
      do ix=nxs2p+1,nxm
      vy(ix,iy)=(psi(ix-1,iy)-psi(ix+1,iy))/ddx
      enddo
C
      enddo
C
      call dtabdx(vy,dvydx)
      call dtabdy(vx,dvxdy)
      call dtabdx(vx,dvxdx)
      call dtabdy(vy,dvydy)
C
	  do iy=iylp,ny
C
		do ix=1,nx
		vy(ix,iy)=0.
		dvxdx(ix,iy)=0.
		dvxdy(ix,iy)=0.
		dvydx(ix,iy)=0.
		dvydy(ix,iy)=0.
		enddo
C
		do ix=1,nxs2
		vx(ix,iy)=-svl
		enddo
C
		do ix=nxs2p,nx
		vx(ix,iy)=svr
		enddo
C
	  enddo
C
      do ix=1,nx
      do iy=1,ny
      vit(ix,iy)=dsqrt(vx(ix,iy)*vx(ix,iy)+vy(ix,iy)*vy(ix,iy))
      enddo
      enddo
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE READING_RESULTS
C
C     READING OF RESULT FILES FORT.IFILE ALLOWING TO RESTART THE
C     COMPUTATION OF A RUN FROM THIS FILE
C
C----------------------------------------------------------------------
      subroutine reading_results(ifile)
C
      include 'parameter'
      include 'common'
      rewind(ifile)
C
      read(ifile)tep
      read(ifile)psi
      read(ifile)vor
C
      close(ifile)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE WRITING_RESULTS
C
C     WRITING OF RESULTS ON FILE FORT.IFILE
C
C----------------------------------------------------------------------
      subroutine writing_results(ifile)
C
      include 'parameter'
      include 'common'
      rewind(ifile)
C
      write(ifile)tep
      write(ifile)psi
      write(ifile)vor
C
      close(ifile)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE Computing_options
C
C     INPUT OF computing options
C
C-----------------------------------------------------------------------
      subroutine computing_options(iprint)
C
      include 'parameter'
      include 'common'
      open(2, file='options_file',form='formatted')
C
      read(2,*)itera
      read(2,*)cglobal
      read(2,*)ip
      read(2,*)
C
      read(2,*)iterm
      read(2,*)cvit
      read(2,*)ipm
      read(2,*)
C
      read(2,*)cour_psi
      read(2,*)itp
      read(2,*)cpsi
      read(2,*)ipp
      read(2,*)
C
      read(2,*)cour_vor
      read(2,*)itv
      read(2,*)cvor
      read(2,*)ipv
      read(2,*)
C
      read(2,*)cour_tep
      read(2,*)itt
      read(2,*)ctep
      read(2,*)ipt
C
      if(iprint.eq.1)then
      print*,'maximum number of global iterations         :',itera
C
      print*,'courant criterion stream-function           :',cour_psi
      print*,'maximum number of iteration stream-function :',itp
      print*,'convergence criterium for stream function   :',cpsi
      print*,'flag impression stream function convergence :',ipp
C
      print*,'courant criterion vorticity                 :',cour_vor
      print*,'maximum number of iteration vorticity       :',itv
      print*,'convergence criterium for vorticity         :',cvor
      print*,'flag impression vortiity convergence        :',ipv
C
      print*,'number stream-function vorticity iterations :',iterm
      print*,'convergence criterium for motion            :',cvit
      print*,'flag impression motion convergence          :',ipm
C
      print*,'courant criterion temperature               :',cour_tep
      print*,'maximum number of iteration temperature     :',itt
      print*,'convergence criterium for temperature       :',ctep
      print*,'flag impression temperature convergence     :',ipt
                      endif
C
      close(2)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE DRAWTAB
C
C EXTRACTION OF (NX,NY) ARRAY DATA FOR NOESYS-TRANSFORM GRAPHIC LIBRARY
C
C-----------------------------------------------------------------------
      subroutine drawtab(ides,x,y,tab,titre)
C
      include 'parameter'
      dimension x(nx),y(ny),tab(nx,ny)
      character*(*) titre
C
      print*,'noesys-transform graphic file fort.',ides,'  ',titre
C
C SEARCHING EXTREMA VALUES
C
      tmax=-1.e+10
      tmin=1.e+10
      do ix =1 ,nx
      do iy =1 ,ny
      if(tab(ix,iy).gt.tmax)tmax=tab(ix,iy)
      if(tab(ix,iy).lt.tmin)tmin=tab(ix,iy)
      enddo
      enddo
C
      write(ides,*)'noesys-transform graphic file'
      write(ides,*)'coordinates x (horizontal) and y (vertical)'
      write(ides,*)'are given in meters'
      write(ides,*)'void'
      write(ides,*)titre
      write(ides,*)ny,nx
      write(ides,*)tmin,tmax
      write(ides,*)(y(iy),iy=ny,1,-1)
      write(ides,*)(x(ix),ix=1,nx)
      do iy=ny,1,-1
      write(ides,*)(tab(ix,iy),ix=1,nx)
      enddo
C
      close(ides)
C
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE TRACER_TRAJECTORIES
C
C THIS SUBROUTINE CALCULATES EXTRACTION OF (NX,NY) ARRAY DATA FOR NOESYS-TRANSFORM GRAPHIC LIBRARY
C
C-----------------------------------------------------------------------
      subroutine tracer_trajectories
C
      include 'parameter'
      include 'common'
      dimension tx(ny),ty(ny)
C
C FIRST STEP :
C CALCULATIONS FOR EACH X POSITIONS OF THE Y POSITIONS CORRESPONDING TO
C THE VALUES OF THE LATERAL STREAM FUNCTION BOUNDARY CONDITIONS
C AND INTEPOLATION AT THESE LOCATIONS OF THE VELOCITY AND TEMPERATURE
C
C LEFT AND RIGHT LATERAL BOUNDARIES
C
      do iy=2,nym
C
	  ypsig(1,iy)=y(iy)
	  teppsig(1,iy)=tep(1,iy)
	  vxpsig(1,iy)=-svl
	  vypsig(1,iy)=0.

	  ypsid(nx,iy)=y(iy)
	  teppsid(nx,iy)=tep(nx,iy)
	  vxpsid(nx,iy)=svr
	  vypsid(nx,iy)=0.
	  enddo
C
C LEFT SIDE OF THE RIDGE (IX=1 TO NXS2)
C
      do iy=2,nym
C
          psig=psi(1,iy)
          do ix=2,nxs2
C
C  CALCULATION OF HEIGHT YPSIG(IX,IY) CORRESPONDING TO THE VALUE PSIG
C  AT OFFSET X(IX)
C

            do iyi=1,ny
            tx(iyi)=psi(ix,iyi)
            ty(iyi)=y(iyi)
            enddo
            call interpolation4p_y(tx,ty,psig,ypsig(ix,iy))
C
C  CALCULATION OF VXPSIG, VALUE OF VX INTERPOLATED AT HEIGHT YPSIG(IX,IY)
C
            do iyi=1,ny
            ty(iyi)=vx(ix,iyi)
            enddo
            call interpolation4p_y(tx,ty,psig,vxpsig(ix,iy))
C
C  CALCULATION OF VYPSIG, VALUE OF VY INTERPOLATED
C
            do iyi=1,ny
            ty(iyi)=vy(ix,iyi)
            enddo
            if(ix.ne.nxs2)then
            call interpolation4p_y(tx,ty,psig,vypsig(ix,iy))
                          else
                          vypsig(ix,iy)=0.
                          endif
C
C  CALCULATION OF TEPPSIG, VALUE OF TEP INTERPOLATED
C
            do iyi=1,ny
            ty(iyi)=tep(ix,iyi)
            enddo
            if(ix.ne.nxs2)then
            call interpolation4p_y(tx,ty,psig,teppsig(ix,iy))
                          else
                          if(iy.le.iyl)teppsig(ix,iy)=tmi
                          if(iy.gt.iyl)teppsig(ix,iy)=tep(ix,iy)
                          endif
C
          enddo
C
      enddo
C
C RIGHT SIDE OF THE RIDGE (IX=NXS2+1 TO NX)
C
      do iy=2,nym
C
          psid=psi(nx,iy)
          do ix=nxs2p,nxm
C
C  CALCULATION OF HEIGHT YPSID(IX,IY) CORRESPONDING TO THE VALUE PSID
C  AT OFFSET X(IX)
C
              do iyi=1,ny
              tx(iyi)=psi(ix,iyi)
              ty(iyi)=y(iyi)
              enddo
              call interpolation4p_y(tx,ty,psid,ypsid(ix,iy))
C
C  CALCULATION OF VXPSID
C
              do iyi=1,ny
              ty(iyi)=vx(ix,iyi)
              enddo
              call interpolation4p_y(tx,ty,psid,vxpsid(ix,iy))
C
C  CALCULATION OF VYPSID
C
              do iyi=1,ny
              ty(iyi)=vy(ix,iyi)
              enddo
              if(ix.ne.nxs2p)then
              call interpolation4p_y(tx,ty,psid,vypsid(ix,iy))
                             else
                             vypsid(ix,iy)=0.
                             endif
C
C  CALCULATION OF TEPPSID
C
              do iyi=1,ny
              ty(iyi)=tep(ix,iyi)
              enddo
              if(ix.ne.nxs2p)then
              call interpolation4p_y(tx,ty,psid,teppsid(ix,iy))
                             else
                             if(iy.le.iyl)teppsid(ix,iy)=tmi
                             if(iy.gt.iyl)teppsid(ix,iy)=tep(ix,iy)
                    if(iy.eq.iyl+2)teppsid(ix,iy)=0.5*(tep(ix,iy)+tmi)
                             endif
C
          enddo
C
      enddo
C
C SECOND STEP :
C CALCULATIONS OF THE TIME SPENT TO FROM THE RIDGE INTRUSION TO THE
C ACTUAL LOCATIONS ON THE TRAJECTORIES
      do iy=2,nym
C
C LEFT SIDE OF THE RIDGE (IX=NXS2 TO 1)
C
          tempspsig(nxs2,iy)=0.
          do ixg=nxs2-1,1,-1
C
      	  dyg=ypsig(ixg,iy)-ypsig(ixg+1,iy)
          distg=dx2+dyg*dyg
C
          vitgx=0.5*(vxpsig(ixg,iy)+vxpsig(ixg+1,iy))
          vitgy=0.5*(vypsig(ixg,iy)+vypsig(ixg+1,iy))
          vpsig=dsqrt(vitgx*vitgx+vitgy*vitgy)

          tempspsig(ixg,iy)=tempspsig(ixg+1,iy)+distg/vpsig
C
          enddo
C
C RIGHT SIDE OF THE RIDGE (IX=NXS2+1 TO NX)
C
          tempspsid(nxs2+1,iy)=0.
          do ixd=nxs2+2,nx
C
          dyd=ypsid(ixd,iy)-ypsid(ixd-1,iy)
          distd=dx2+dyd*dyd
C
          vitdx=0.5*(vxpsid(ixd,iy)+vxpsid(ixd-1,iy))
          vitdy=0.5*(vypsid(ixd,iy)+vypsid(ixd-1,iy))
          vpsid=dsqrt(vitdx*vitdx+vitdy*vitdy)
C
          tempspsid(ixd,iy)=tempspsid(ixd-1,iy)+distd/vpsid
C
          enddo
C
      enddo
C
      print*
      print*,'end of tracer_trajectories'
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE COOLING_RATES_ICR
C
C THIS SUBROUTINE CALCULATES THE COOLING RATE OF GABBROS WITHIN THE
C OPENING-CLOSURE TEMPERATURE RANGE CORRESPONDING TO THE
C IGNEOUS COOLING RATES
C
C-----------------------------------------------------------------------
      subroutine cooling_rates_ICR
C
      include 'parameter'
      include 'common'
      dimension tx(nx),ty(nx)
C
C LEFT SIDE OF THE RIDGE (IX=NXS2 TO 1)
C
      do iy=2,nym
      if(teppsig(nxs2,iy).le.tcicr)then
                                   trgicr(iy)=0.
                                   goto 99
                                   else
                                   tdeb=0.
                                   do ix=1,nxs2
                                   tx(ix)=teppsig(ix,iy)
                                   ty(ix)=tempspsig(ix,iy)
                                   enddo
                                   ideb=1
                                   ifin=nxs2
                call interpolation4p_x(ideb,ifin,tx,ty,tcicr,tfin,0)
                                   trgicr(iy)=(toicr-tcicr)/(tfin-tdeb)
                                   endif
C
   99 continue
C
      enddo
C
C RIGHT SIDE OF THE RIDGE (IX=NXS2+1 TO NX)
C
      do iy=2,nym
      if(teppsid(nxs2+1,iy).le.tcicr)then
                                     trdicr(iy)=0.
                                     goto 98
                                     else
                                     tdeb=0.
                                     do ix=nxs2+1,nx
                                     tx(ix)=teppsid(ix,iy)
                                     ty(ix)=tempspsid(ix,iy)
                                     enddo
                                     ideb=nxs2+1
                                     ifin=nx
                 call interpolation4p_x(ideb,ifin,tx,ty,tcicr,tfin,0)
                            trdicr(iy)=(toicr-tcicr)/(tfin-tdeb)
                                     endif
   98 continue
C
      enddo
C
      cvsparan=365.25*24.*3600.
      do iy=2,nym
      trgicr(iy)=trgicr(iy)*cvsparan
      trdicr(iy)=trdicr(iy)*cvsparan
      enddo
C
C WRITING THE RESULTS ON FILE FORT.206
C
      write(206,*)'Cooling rates ICR'
      write(206,*)'Height(m)',' ','ICR-g (K/y)',' ','ICR-d (K/y)'
      do iy=2,nym
      write(206,*)y(iy),trgicr(iy),trdicr(iy)
      enddo
C
      print*
      print*,'end of cooling_rates_ICR'
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE COOLING_RATES_SCR
C
C THIS SUBROUTINE CALCULATES THE COOLING RATE OF GABBROS WITHIN THE
C OPENING-CLOSURE TEMPERATURE RANGE CORRESPONDING TO THE
C SUBSOLIDUS COOLING RATES
C
C-----------------------------------------------------------------------
      subroutine cooling_rates_SCR
C
      include 'parameter'
      include 'common'
      dimension tx(nx),ty(nx)
C
C LEFT SIDE OF THE RIDGE (IX=NXS2 TO 1)
C
      do iy=2,nym
      if(teppsig(nxs2,iy).le.tcscr)then
                                   trgscr(iy)=0.
                                   goto 99
                                   else
                                   do ix=1,nxs2
                                   tx(ix)=teppsig(ix,iy)
                                   ty(ix)=tempspsig(ix,iy)
                                   enddo
                                   ideb=1
                                   ifin=nxs2
                 call interpolation4p_x(ideb,ifin,tx,ty,toscr,tdeb,0)
                                   endif
C
      if(teppsig(1,iy).gt.tcscr)then
                                trgscr(iy)=0.
                                goto 99
                                else
                                do ix=1,nxs2
                                tx(ix)=teppsig(ix,iy)
                                ty(ix)=tempspsig(ix,iy)
                                enddo
                                ideb=1
                                ifin=nxs2
                 call interpolation4p_x(ideb,ifin,tx,ty,tcscr,tfin,0)
                                endif
C
      trgscr(iy)=(toscr-tcscr)/(tfin-tdeb)
C
   99 continue
      enddo
C
C RIGHT SIDE OF THE RIDGE (IX=NXS2+1 TO NX)
C
      do iy=2,nym
      if(teppsid(nxs2+1,iy).le.tcscr)then
                                     trdscr(iy)=0.
                                     goto 98
                                     else
                                     do ix=nxs2+1,nx
                                     tx(ix)=teppsid(ix,iy)
                                     ty(ix)=tempspsid(ix,iy)
                                     enddo
                                     ideb=nxs2+1
                                     ifin=nx
                   call interpolation4p_x(ideb,ifin,tx,ty,toscr,tfin,0)
                                     endif
C
      if(teppsid(nx,iy).gt.tcscr)then
                                 trdscr(iy)=0.
                                 goto 98
                                 else
                                 do ix=nxs2+1,nx
                                 tx(ix)=teppsid(ix,iy)
                                 ty(ix)=tempspsid(ix,iy)
                                 enddo
                                 ideb=nxs2+1
                                 ifin=nx
                   call interpolation4p_x(ideb,ifin,tx,ty,tcscr,tfin,0)
                                 endif
C
      trdscr(iy)=(toscr-tcscr)/(tfin-tdeb)
   98 continue
      enddo
C
      cvsparan=365.25*24.*3600.
      do iy=2,nym
      trgscr(iy)=trgscr(iy)*cvsparan
      trdscr(iy)=trdscr(iy)*cvsparan
      enddo
C
C WRITING THE RESULTS ON FILE FORT.207
C
      write(207,*)'cooling rates scr'
      write(207,*)'Height(m)',' ','SCR-g (K/y)',' ','SCR-d (K/y)'
      do iy=2,nym
      write(207,*)y(iy),trgscr(iy),trdscr(iy)
      enddo
C
      print*
      print*,'end of cooling_rates_SCR'
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE TEMPERATURE_HISTORY
C
C THIS SUBROUTINE CALCULATES AND DRAW THE HISTORY OF TEMPERATURE
C ALONG THE TRACER TRAJECTORIES
C
C-----------------------------------------------------------------------
      subroutine temperature_history
      include 'parameter'
      include 'common'
      dimension tig(0:nstep),tid(0:nstep)
      dimension tepig(0:nstep,2:nym),tepid(0:nstep,2:nym)
      dimension tepigd(-nstep:nstep,2:nym),tigd(-nstep:nstep)
      dimension tx(nx),ty(nx)
C
      spa=3600.d0*24.d0*365.25d0
      tmaxg=0.d0
      tmaxd=0.d0
C
      xcma=0.2d0
C
      do iy=2,nym
      if(tempspsid(nx,iy).gt.tmaxd)tmaxd=tempspsid(nx,iy)
      if(tempspsig(1,iy).gt.tmaxg)tmaxg=tempspsig(1,iy)
      enddo
C
C TIME CONVERSION FROM S TO MILLION YEARS
C TEMPERATURE FROM K TO C
C
      tmaxd=tmaxd/spa/1.d+06
      tmaxg=tmaxg/spa/1.d+06
C
      do iy=2,nym
C
          do ix=1,nxs2-1
          tempspsig(ix,iy)=tempspsig(ix,iy)/spa/1.d+06
          enddo
C
          do ix=nxs2+2,nx
          tempspsid(ix,iy)=tempspsid(ix,iy)/spa/1.d+06
          enddo
C
      enddo
C
      do iy=1,ny
C
          do ix=nxs2+1,nx
          teppsid(ix,iy)=teppsid(ix,iy)-273.d0
          enddo
C
          do ix=1,nxs2
          teppsig(ix,iy)=teppsig(ix,iy)-273.d0
          enddo
C
      enddo
C
C TIMES FOR WHICH TEMPERATURES WILL BE INTERPOLATED
C
         tig(0)=0.
         tid(0)=0.
         do it=1,nstep
         tig(it)=xcma*dfloat(it)
         tid(it)=xcma*dfloat(it)
         enddo
C
         do it=-nstep,nstep
         tigd(it)=xcma*dfloat(it)
         enddo
C
C INTERPOLATION OF TEMPERATURES
C
C
C LEFT SIDE OF THE RIDGE (IX=NXS2 TO 1)
C
      do iy=2,nym
      tepig(0,iy)=teppsig(nxs2,iy)
C
          do ix=1,nxs2
          tx(ix)=tempspsig(ix,iy)
          ty(ix)=teppsig(ix,iy)
          enddo
C
          do ii=1,nstep
          ideb=1
          ifin=nxs2
      call interpolation4p_x(ideb,ifin,tx,ty,tig(ii),tepig(ii,iy),0)
          enddo
C
      enddo
C
C RIGHT SIDE OF THE RIDGE (IX=NXS2+1 TO NX)
C
      do iy=2,nym
      tepid(0,iy)=teppsid(nxs2p,iy)
C
          do ix=nxs2p,nx
          tx(ix)=tempspsid(ix,iy)
          ty(ix)=teppsid(ix,iy)
          enddo
C
          do ii=1,nstep
          ideb=nxs2p
          ifin=nx
      call interpolation4p_x(ideb,ifin,tx,ty,tid(ii),tepid(ii,iy),0)
          enddo
C
      enddo
C
C     FUSION OF LEFT AND RIGHT RESULTS IN ONLY ONE ARRAY TEPIGD
C     TEPIGD  TEMPERATURE (TEP) INTERPOLATED (I) LEFT (G) RIGHT (D)
C
      do iy=2,nym
      tepigd(0,iy)=0.5*(tepig(0,iy)+tepid(0,iy))
C
          do is=1,nstep
          ig=-is
          tepigd(ig,iy)=tepig(is,iy)
          id=is
          tepigd(id,iy)=tepid(is,iy)
          enddo
C
      enddo
C
C EXTRACTION OF RESULTS FOR NOESYS-TRANSFORM GRAPHIC LIBRARY FORT.208
C
      tmin=tsurf-273.
      tmax=tmi-273.
C
      ides=208
C
      write(ides,*)'array tep(y,t) noesys_transform'
      write(ides,*)'horizontal: time (My); vertical: height (m)'
      write(ides,*)'void'
      write(ides,*)'void'
      write(ides,*)nym2,2*nstep+1
      write(ides,*)tmin,tmax
      write(ides,*)(y(iy),iy=nym,2,-1)
      write(ides,*)(tigd(it),it=-nstep,nstep)
      do iy=nym,2,-1
      write(ides,*)(tepigd(it,iy),it=-nstep,nstep)
      enddo
      close(ides)
C
      print*
      print*,'end of temperature_history'
      return
      end
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C SUBROUTINE convergence_xy
C
C THIS SUBROUTINE tests the convergence of computation on a 2D array
C
C-----------------------------------------------------------------------
      subroutine convergence_xy(it,ip,t,tp,cc,titre,iconv)
C
      include 'parameter'
      dimension t(nx,ny),tp(nx,ny)
      character*(*)titre
C
      iconv=0
      if(it.eq.1)return
      tm=0.
      ecm=0.
C
      do ix=1,nx
C
          do iy=1,ny
          xm=abs(t(ix,iy))
          if(xm.gt.tm)tm=xm
          ec=abs(t(ix,iy)-tp(ix,iy))
          if(ec.gt.ecm)ecm=ec
          enddo
C
      enddo
C
      e=ecm/tm
      if(ip.eq.1)print*,'ce ',titre,' : ',it,e
      if(e.le.cc)iconv=1
C
      return
      end

